/*****************************************************************************
*	Filename:  PCSPEECH.C                                                *
******************************************************************************
*	Author:		Rodger Richey                                        *
*	Title:		Senior Applications Engineer                         *
*	Company:	Microchip Technology Incorporated                    *
*	Revision:	0                                                    *
*	Date:		1-11-96                                              *
*	Compiled using Borland C+ Version 3.1                                *
******************************************************************************
*	Include files:                                                       *
*		stdio.h - Standard input/output header file                  *
*		stdlib.h - Standard library header file                      *
*		string.h - Standard string header file                       *
*		pcadpcm.h - ADPCM related information header file (Rev0)     *
******************************************************************************
*	Usage:                                                               *
*		ADPCM Encode - pcspeech e <infile> <outfile>                 *
*		               <infile> is a 16-bit raw speech file          *
*			       <outfile> contains the ADPCM codes            *
*		ADPCM Decode - pcspeech d <infile> <outfile>                 *
*			       <infile> contains the ADPCM codes             *
*			       <outfile> is a 16-bit raw speech file         *
******************************************************************************
*	This file contains the code to:                                      *
*		- Open the input and output files                            *
*		- Read data from the input file                              *
*		- Call the encode/decode routines                            *
*		- Write data to the output file                              *
*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <io.h>
#include <conio.h>
#include <string.h>
#include "pcadpcm.h"

/*****************************************************************************
*	Usage - this routine prints a how to message for the pcspeech prgm   *
******************************************************************************
*	Input variables:                                                     *
*		None                                                         *
*	Output variables:                                                    *
*		None                                                         *
*****************************************************************************/
void Usage(void)
{
	printf("ADPCM Compression/Decompression -- usage:\n");
	printf("\tEncoder = pcspeech e infile outfile\n");
	printf("\tDecoder = pcspeech d infile outfile\n");
	exit(1);
}

/*****************************************************************************
*	main - controls file I/O and ADPCM calls                             *
******************************************************************************
*	Input variables:                                                     *
*		int argc - number of arguements in argv                      *
*		char **argv - pointer to an array of strings                 *
*	Return variables:                                                    *
*		None                                                         *
*****************************************************************************/
void main(
	int   argc,
	char  **argv)
{
	int 			which;
	short                   sample;
	unsigned char           code;
	int                     n,i;
	struct ADPCMstate       state;
	int			handlein,handleout;
	long			filesize;
	long			bytes=0;
	int			percent=0,lastp=0;
	char			filein[20],fileout[20];
	FILE			*fpin;
	FILE			*fpout;

	state.prevsample=0;
	state.previndex=0;

	/* Determine if this is an encode or decode operation */
	if(argc <= 1)
		Usage();
	else if( strcmp(argv[1],"e")==0 || strcmp(argv[1],"E")==0 )
		which = 0;
	else if( strcmp(argv[1],"d")==0 || strcmp(argv[1],"D")==0 )
		which = 1;
	argc--;
	argv++;

	/* Open input file for processing */
	if(argc <= 1)
		Usage();
	else if( (fpin=fopen(argv[1],"rb"))==NULL)
	{
		printf("ADPCM Compression/Decompression\n");
		printf("ERROR: Could not open %s for input\n",argv[1]);
		exit(1);
	}
	strcpy(filein,argv[1]);
	argc--;
	argv++;

	/* Open output file */
	if(argc <= 1)
	{
		fclose(fpin);
		Usage();
	}
	else if( (fpout=fopen(argv[1],"wb"))==NULL)
	{
		fclose(fpin);
		printf("ADPCM Compression/Decompression\n");
		printf("ERROR: Could not open %s for output\n",argv[1]);
		exit(1);
	}

//	fp=fopen("decode.dat","w");

	strcpy(fileout,argv[1]);

	handlein = fileno(fpin);
	handleout = fileno(fpout);
	filesize = filelength(handlein);
	_setcursortype(_NOCURSOR);
	clrscr();
	printf("%c",201);
	for(i=0;i<52;i++)
		printf("%c",205);
	printf("%c\n",187);
	printf("%c           ADPCM Compression/Decompression          %c\n",186,186);
	printf("%c",204);
	for(i=0;i<52;i++)
		printf("%c",205);
	printf("%c\n",185);

	// ADPCM Decoder selected
	if(which)
	{
		printf("%c                 ADPCM Decompression                %c\n",186,186);
		printf("%c 0        20        40        60        80      100 %c\n",186,186);
		printf("%c ",186);
		for(i=0;i<50;i++)
			printf("%c",176);
		printf(" %c\r%c ",186,186);
		highvideo();

		/* Read and unpack input codes and process them */
		while (fread(&code, sizeof (unsigned char), 1, fpin) == 1)
		{
			// Send the upper 4-bits of code to decoder
			sample = ADPCMDecoder((code>>4)&0x0f, &state);
			// Write sample for upper 4-bits of code
			fwrite(&sample, sizeof(short), 1, fpout);

			// Send the lower 4-bits of code to decoder
			sample = ADPCMDecoder(code&0x0f,&state);
			// Write sample for lower 4-bits of code
			fwrite(&sample,sizeof(short),1,fpout);
			bytes++;
			percent=(bytes*100)/filesize;
			if(percent%2&&percent>lastp)
			{
				cprintf("%c",178);
				lastp=percent;
			}
		}
		normvideo();
		printf("\n");
	}
	// ADPCM Encoder selected
	else
	{
		filesize/=2;
		printf("%c                  ADPCM Compression                 %c\n",186,186);
		printf("%c 0        20        40        60        80      100 %c\n",186,186);
		printf("%c ",186);
		for(i=0;i<50;i++)
			printf("%c",176);
		printf(" %c\r%c ",186,186);
		highvideo();
		/* Read input file and process */
		while (fread(&sample, sizeof(short), 1, fpin) == 1)
		{
			// Encode sample into lower 4-bits of code
			code = ADPCMEncoder(sample,&state);
			// Move ADPCM code to upper 4-bits
			code = (code << 4) & 0xf0;

			bytes++;
			percent=(bytes*100)/filesize;
			if(percent%2&&percent>lastp)
			{
				cprintf("%c",178);
				lastp=percent;
			}

			// Read new sample from file
			if(fread(&sample,sizeof(short),1,fpin)==0)
			{
				// No more samples, write code to file
				fwrite(&code,sizeof(unsigned char),1,fpout);
				break;
			}
			// Encode sample and save in lower 4-bits of code
			code |= ADPCMEncoder(sample,&state);
			// Write code to file, code contains 2 ADPCM codes
			fwrite(&code, sizeof (unsigned char), 1, fpout);

			bytes++;
			percent=(bytes*100)/filesize;
			if(percent%2&&percent>lastp)
			{
				cprintf("%c",178);
				lastp=percent;
			}
		}
		normvideo();
		printf(" %c\n",186);
	}

	_setcursortype(_NORMALCURSOR);

	printf("%c",204);
	for(i=0;i<52;i++)
		printf("%c",205);
	printf("%c\n",185);
	printf("%c Input file size:    %12s =>  %7ld bytes %c\n",186,filein,filelength(handlein),186);
	printf("%c Output file size:   %12s =>  %7ld bytes %c\n",186,fileout,filelength(handleout),186);
	if(!which)
		printf("%c Compression Ratio => %2.1f to 1                      %c\n",186,(float)filelength(handlein)/(float)filelength(handleout),186);
	printf("%c",200);
	for(i=0;i<52;i++)
		printf("%c",205);
	printf("%c\n\n",188);

	fclose(fpin);
	fclose(fpout);
}
